﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『ボタンにカラーボックスを表示する方法』
'  │・予めBitmapを作成しておき、そこに任意の色で四角形を描き、それを[Button]コントロールの[Image]プロパティに入れて再描画しているだけ。
'  │・ただし、書き換えるたびに[Image]プロパティに登録しなおさないと、なぜか反映されないみたい。
'  │
'  └────────────





Imports System.ComponentModel

''' <summary>
''' カラーボックス付きボタン
''' </summary>
''' <remarks>Buttonコントロールを継承する</remarks>
Public Class ColorBoxButton

    Inherits Button


    Private Bmp As Bitmap, Grp As Graphics

    Private _BoxSize As Size = New Size(Me.FontHeight, Me.FontHeight) 'カラーボックスのサイズ
    Private _BoxForeColor As Color = Color.White 'カラーボックスに引く斜線の色
    Private _BoxBackColor As Color = Color.Black 'カラーボックスの色


    Public Sub New()

        MyBase.New()
        MakeColorBox()
        TextImageRelation = Windows.Forms.TextImageRelation.ImageBeforeText

    End Sub

    ''' <summary>
    ''' カーラーボックスのサイズ
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("カラーボックス")> _
    <Description("カラーボックスのサイズを指定します。")> _
    Public Property ColorBoxSize() As Size

        Get
            Return _BoxSize
        End Get

        Set(ByVal value As Size)
            _BoxSize = value
            MakeColorBox()
        End Set

    End Property

    ''' <summary>
    ''' カラーボックスの前景色
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>前景色とは、カラーボックスに引く斜線の色のこと。</remarks>
    <Category("カラーボックス")> _
    <Description("カラーボックスの前景色を指定します。")> _
    Public Property BoxForeColor() As Color

        Get
            Return _BoxForeColor
        End Get

        Set(ByVal value As Color)
            _BoxForeColor = value
            DrawColorBox()
            Refresh()
        End Set

    End Property

    ''' <summary>
    ''' カラーボックスの背景色
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>背景色とは、カラーボックスの色のこと。</remarks>
    <Category("カラーボックス")> _
    <Description("カラーボックスの背景色を指定します。")> _
    Public Property BoxBackColor() As Color

        Get
            Return _BoxBackColor
        End Get

        Set(ByVal value As Color)
            _BoxBackColor = value
            DrawColorBox()
            Refresh()
        End Set

    End Property

    ''' <summary>
    ''' カラーボックスを作る
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub MakeColorBox()

        Bmp = New Bitmap(ColorBoxSize.Width, ColorBoxSize.Height)
        Grp = Graphics.FromImage(Bmp)

        DrawColorBox()

    End Sub

    ''' <summary>
    ''' カラーボックスを描画する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DrawColorBox()

        With Grp
            .Clear(BoxBackColor)                                                  'カラーボックスの色でクリアする。
            .DrawLine(New Pen(BoxForeColor), 0, 0, Bmp.Width - 1, Bmp.Height - 1) '斜線を引く。
            .DrawRectangle(Pens.Black, 0, 0, Bmp.Width - 1, Bmp.Height - 1)       'カラーボックスの枠を描く。
        End With

        Image = Bmp '画像を再描画するたびに[Image]プロパティに登録し直さないと、コントロールに反映されないみたい。

    End Sub

End Class
