﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『[Keys]の内容のXML化について』
'  │・[Keys]に修飾子付きキーコードを設定してXML化すると、余計なものが出力される。
'  │例）[Keys.Alt Or Keys.Up] → "RButton MButton XButton2 Space Next PageDown Home Up Alt"（"RButton MButton XButton2 Space Next PageDown Home"が不用）
'  │・これは、キーコードの重複が原因だと思われる。このままでも、不用な部分を削除しても、キーコード自体は全く同じ。
'  │
'  └────────────





''' <summary>
''' ファイル管理
''' </summary>
''' <remarks></remarks>
Module FileManager

    ''' <summary>設定ファイル名</summary>
    Public ReadOnly SettingFileName As String = System.IO.Path.GetFullPath(".\Setting.xml")

    ''' <summary>記録ファイル名</summary>
    Public ReadOnly ResumeFileName As String = System.IO.Path.GetFullPath(".\Resume.xml")


    '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    ''' <summary>設定内容</summary>
    Public SettingData As New SettingParameter

    ''' <summary>記録内容</summary>
    Public ResumeData As New ResumeParameter


    '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


#Region "Load"

    ''' <summary>
    ''' XMLファイルからの復元
    ''' </summary>
    ''' <param name="FileName">ファイル名</param>
    ''' <param name="LoadObject">復元先</param>
    ''' <remarks></remarks>
    Private Sub FileLoadBase(FileName As String, ByRef LoadObject As Object)

        '=== XMLファイルから読み込む ===
        If System.IO.File.Exists(FileName) Then

            Try

                '--- XMLファイルから読み込み、逆シリアル化する ---
                Dim serializer As New System.Xml.Serialization.XmlSerializer(LoadObject.GetType) 'XMLシリアル化機構を作成する。
                Using streamer As New System.IO.FileStream(FileName, System.IO.FileMode.Open) 'ファイルを開く。
                    LoadObject = CTypeDynamic(serializer.Deserialize(streamer), LoadObject.GetType) 'ファイルから復元する。
                End Using

            Catch ex As Exception

                Throw 'エラーを再スロー

            End Try

        End If

    End Sub

    ''' <summary>
    ''' 設定内容を復元する
    ''' </summary>
    ''' <param name="LoadObject">読込先</param>
    ''' <returns>不正値修正結果</returns>
    ''' <remarks></remarks>
    Public Function SettingLoad(ByRef LoadObject As SettingParameter) As String

        Try
            FileLoadBase(SettingFileName, LoadObject)
        Catch ex As Exception
            Throw
        End Try

        '=== 不正値修正 ===
        Return InjusticeValueCheck()

    End Function

    ''' <summary>
    ''' 記録内容を復元する
    ''' </summary>
    ''' <param name="LoadObject">読込先</param>
    ''' <remarks></remarks>
    Public Sub ResumeLoad(ByRef LoadObject As ResumeParameter)

        Try
            FileLoadBase(ResumeFileName, LoadObject)
        Catch ex As Exception
            Throw
        End Try

    End Sub

    ''' <summary>
    ''' 不正値の確認と修正
    ''' </summary>
    ''' <remarks></remarks>
    Public Function InjusticeValueCheck() As String

        'ファイルに記述がない場合は、未設定（変数等の初期値）の状態で起動するため、未設定では不都合なものを修正する。
        'ちなみに、ファイルの記述が意味不明な場合は、XML解析の段階でエラーとなり、コードに記述したデフォルト値の状態で起動する。要するに、ファイルがなかった場合と同じ。

        '=== 設定値の修正 ===
        Dim errorItem As String = "" '問題があった項目名
        Dim DefaultStteing As New SettingParameter 'デフォルト値

        With SettingData

            With .Lupe

                If .RefreshInterval < 1 OrElse .RefreshInterval > 1000 Then '間隔が0ms以下または1001ms以上（1000msは設定画面での調整最大値）
                    .RefreshInterval = DefaultStteing.Lupe.RefreshInterval
                    errorItem &= "|範囲外：<拡大鏡>-<更新間隔>|"
                End If

                With .View

                    With .Area
                        If .StandardSize IsNot Nothing Then
                            For i As Integer = .StandardSize.Count - 1 To 0 Step -1
                                If .StandardSize(i) < 3 OrElse .StandardSize(i) > 50 Then '許容範囲越え
                                    .StandardSize.RemoveAt(i)
                                    errorItem &= "|範囲外：<拡大鏡>-<ビューア>-<拡大範囲>-<規定サイズ> (" & i.ToString & ")|"
                                End If
                            Next
                        End If

                        If .StandardSize Is Nothing OrElse .StandardSize.Count = 0 Then '選択肢がない（許容範囲越えで削除されて選択肢がなくなってしまう場合に備えて後に行う。）
                            .StandardSize = DefaultStteing.Lupe.View.Area.StandardSize
                            errorItem &= "|未記述：<拡大鏡>-<ビューア>-<拡大範囲>-<規定サイズ>|"
                        End If

                        If .SelectWidthSizeIndex < 0 OrElse .SelectWidthSizeIndex >= .StandardSize.Count Then '選択番号が選択肢の範囲外
                            .SelectWidthSizeIndex = .StandardSize.Count \ 2 '中間的な値を期待して、真ん中の選択肢を設定する。
                            errorItem &= "|範囲外：<拡大鏡>-<ビューア>-<拡大範囲>-<横幅選択番号>|"
                        End If

                        If .SelectHeightSizeIndex < 0 OrElse .SelectHeightSizeIndex >= .StandardSize.Count Then '選択番号が選択肢の範囲外
                            .SelectHeightSizeIndex = .StandardSize.Count \ 2 '中間的な値を期待して、真ん中の選択肢を設定する。
                            errorItem &= "|範囲外：<拡大鏡>-<ビューア>-<拡大範囲>-<縦幅選択番号>|"
                        End If
                    End With

                    With .Magnification
                        If .StandardChoices IsNot Nothing Then
                            For i As Integer = .StandardChoices.Count - 1 To 0 Step -1
                                If .StandardChoices(i) < 2 OrElse .StandardChoices(i) > 50 Then '許容範囲越え
                                    .StandardChoices.RemoveAt(i)
                                    errorItem &= "|範囲外：<拡大鏡>-<ビューア>-<倍率>-<規定値> (" & i.ToString & ")|"
                                End If
                            Next
                        End If

                        If .StandardChoices Is Nothing OrElse .StandardChoices.Count = 0 Then '選択肢がない（許容範囲越えで削除されて選択肢がなくなってしまう場合に備えて後に行う。）
                            .StandardChoices = DefaultStteing.Lupe.View.Magnification.StandardChoices
                            errorItem &= "|未記述：<拡大鏡>-<ビューア>-<倍率>-<規定値>|"
                        End If

                        If .SelectIndex < 0 OrElse .SelectIndex >= .StandardChoices.Count Then '選択番号が選択肢の範囲外
                            .SelectIndex = .StandardChoices.Count \ 2 '中間的な値を期待して、真ん中の選択肢を設定する。
                            errorItem &= "|範囲外：<拡大鏡>-<ビューア>-<倍率>-<選択番号>|"
                        End If
                    End With

                End With

            End With

            With .CommandPad

                If .Opacity < 0.1 Then '許容範囲越え
                    .Opacity = DefaultStteing.CommandPad.Opacity '《Opacity》は、範囲外の数値を入れてもエラーを出さない（自動的に0または1になる）ので対処を必要としないが、透明では使えないので下限のみ設ける。
                    errorItem &= "|範囲外：<コマンドパッド>-<不透明度>|"
                End If

            End With

        End With

        Return errorItem.Replace("||", vbCrLf).Replace("|", "")

    End Function

#End Region

#Region "Save"

    ''' <summary>
    ''' XMLファイルへの保存
    ''' </summary>
    ''' <param name="FileName">ファイル名</param>
    ''' <param name="SaveObject">保存元</param>
    ''' <remarks></remarks>
    Private Sub FileSaveBase(FileName As String, SaveObject As Object)

        '=== XMLファイルに書き込む ===
        Try

            '--- シリアル化し、XMLファイルに保存する ---
            Dim serializer As New System.Xml.Serialization.XmlSerializer(SaveObject.GetType) 'XMLシリアル化機構を作成する。
            Using streamer As New System.IO.FileStream(FileName, System.IO.FileMode.Create) 'ファイルを開く。
                serializer.Serialize(streamer, SaveObject) '用意したファイルに書き出す。
            End Using

        Catch ex As Exception

            Throw 'エラーを再スロー

        End Try

    End Sub

    ''' <summary>
    ''' 設定内容を保存する
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub SettingSave()

        Try
            FileSaveBase(SettingFileName, SettingData)
        Catch ex As Exception
            Throw
        End Try

    End Sub

    ''' <summary>
    ''' 記録内容を保存する
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ResumeSave()

        Try
            FileSaveBase(ResumeFileName, ResumeData)
        Catch ex As Exception
            Throw
        End Try

    End Sub

#End Region

End Module





'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





''' <summary>
''' 設定項目
''' </summary>
''' <remarks></remarks>
<Serializable()> _
<System.Xml.Serialization.XmlRoot("HELuPERの設定内容")> _
Public Class SettingParameter

    ''' <summary>
    ''' 複製する
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function Clone() As SettingParameter

        'オブジェクトの内容を、メモリ上に一旦バイナリ形式でシリアル化し、再び逆シリアル化することで、複製してしまう。（参考：http://d.hatena.ne.jp/tekk/20100131/1264913887）
        '型を[Object]にすれば、いろんな形に対応できる。
        '速度的な利点はないかもしれないけど、抜群のお手軽さがある。特に、自作クラスなど、どんな複雑な物でも一発で簡単に複製できてしまう魔法のコード。全項目を一つずつコピーするなんて超面倒な作業にオサラバ。
        '（あまりにも入れ子状態が深すぎると駄目なのかも・・・http://support.microsoft.com/default.aspx?scid=kb;JA;320079）
        '※複製する全てのクラスの頭に、"<Serializable()>"を付けなければならない。

        Dim twinObject As SettingParameter '複製したオブジェクト

        Try

            Dim formatter As New System.Runtime.Serialization.Formatters.Binary.BinaryFormatter() 'バイナリ形式シリアル化機構を作成する。
            Using streamer As New System.IO.MemoryStream() 'メモリを用意する。
                formatter.Serialize(streamer, Me) '用意したメモリに書き出す。
                streamer.Position = 0                        '┬ メモリから復元する。
                twinObject = formatter.Deserialize(streamer) '┘
            End Using

        Catch ex As Exception

            Throw

        End Try

        Return twinObject

    End Function



    '============================================================================================================================



    ''' <summary>
    ''' 拡大鏡に関する設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("拡大鏡")> _
    Public Property Lupe() As LupeParameter = New LupeParameter With {.RefreshInterval = 100}

    ''' <summary>
    ''' 操作キーに関する設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("操作キー")> _
    Public Property OperationKey() As OperationKeyParameter = New OperationKeyParameter With {.CommandPadCallKey = Keys.Control Or Keys.Alt Or Keys.LWin, .ShiftOpacity = 0.0R, .ControlOpacity = 0.3R}

    ''' <summary>
    ''' コマンドパッドに関する設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("コマンドパッド")> _
    Public Property CommandPad() As New CommandPadParameter





    '||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





    ''' <summary>
    ''' 拡大鏡に関する設定項目
    ''' </summary>
    ''' <remarks></remarks>
    <Serializable()> _
    Public Class LupeParameter

        ''' <summary>
        ''' 更新する間隔を設定・取得します
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("更新間隔")> _
        Public Property RefreshInterval() As Integer

        ''' <summary>
        ''' ビューアに関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("ビューア")> _
        Public Property View() As New ViewerParameter

        ''' <summary>
        ''' 情報パネルに関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("情報パネル")> _
        Public Property InformationPanel() As InformationPanelParameter = New InformationPanelParameter With {.Style = FormInformationPanel.LooksStyle.Bar}

        ''' <summary>
        ''' マウスカーソル回避に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("回避")> _
        Public Property Avoid() As AvoidParameter = New AvoidParameter With {.Mode = FormMain.AvoidType.RightTriangle, .Margin = 64}



        '============================================================================================================================



        ''' <summary>
        ''' マウスカーソル回避に関する設定項目
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()> _
        Public Class AvoidParameter

            ''' <summary>
            ''' 回避方法を設定・取得します
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("方法")> _
            Public Property Mode() As FormMain.AvoidType

            ''' <summary>
            ''' マウスの接近を許す距離を設定・取得します
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks>負数になったり[Integer]の最大値を超えたりしたら困るし、それほど大きい数字も必要ないから[UShort]型</remarks>
            <System.Xml.Serialization.XmlElement("許容範囲")> _
            Public Property Margin() As UShort

        End Class

    End Class





    '||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





    ''' <summary>
    ''' ビューアに関する設定項目
    ''' </summary>
    ''' <remarks></remarks>
    <Serializable()> _
    Public Class ViewerParameter

        ''' <summary>
        ''' 拡大範囲に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("拡大範囲")> _
        Public Property Area() As AreaParameter = New AreaParameter With {.StandardSize = New Integer() {7, 15, 31}.ToList, .SelectWidthSizeIndex = 1, .SelectHeightSizeIndex = 1}

        ''' <summary>
        ''' 倍率に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("倍率")> _
        Public Property Magnification() As MagnificationParameter = New MagnificationParameter With {.StandardChoices = New Integer() {4, 8, 12, 16}.ToList, .SelectIndex = 1}

        ''' <summary>
        ''' 描画に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("描画")> _
        Public Property Draw As New DrawParameter



        '============================================================================================================================



        ''' <summary>
        ''' 拡大範囲に関する設定項目
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()> _
        Public Class AreaParameter

            ''' <summary>
            ''' 拡大範囲サイズの規定値
            ''' </summary>
            ''' <remarks>中心点がなければならないので必ず奇数</remarks>
            <System.Xml.Serialization.XmlElement("規定サイズ")> _
            Public StandardSize As List(Of Integer)

            ''' <summary>
            ''' 選択している横幅番号
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("横幅選択番号")> _
            Public SelectWidthSizeIndex As Integer

            ''' <summary>
            ''' 選択している縦幅番号
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("縦幅選択番号")> _
            Public SelectHeightSizeIndex As Integer

        End Class



        '----------------------------------------------------------------------------------------------------------------------------



        ''' <summary>
        ''' 拡大倍率に関する設定項目
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()> _
        Public Class MagnificationParameter

            ''' <summary>
            ''' 規定値
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("規定値")> _
            Public StandardChoices As List(Of Integer)

            ''' <summary>
            ''' 選択している倍率番号
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("選択番号")> _
            Public SelectIndex As Integer

        End Class



        '----------------------------------------------------------------------------------------------------------------------------



        ''' <summary>
        ''' 描画に関する設定項目
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()> _
        Public Class DrawParameter

            ''' <summary>
            ''' カーソルマーカー
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("カーソルマーカー")> _
            Public Property CursorMarker() As SwitchColor = New SwitchColor With {.Visible = True, .ForeColor = Color.Red, .BackColor = Color.FromArgb(&HAA, Color.Black)}

            ''' <summary>
            ''' 格子線
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("格子線")> _
            Public Property Grid() As SwitchColor = New SwitchColor With {.Visible = True, .ForeColor = Color.Black}

            ''' <summary>
            ''' 下地
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("下地")> _
            Public Property Foundation() As SwitchColor = New SwitchColor With {.Visible = True, .ForeColor = Color.White, .BackColor = Color.Black}



            '============================================================================================================================



            ''' <summary>
            ''' スイッチ付きの前背景色
            ''' </summary>
            ''' <remarks></remarks>
            <Serializable()> _
            Public Class SwitchColor

                ''' <summary>
                ''' 有効/無効
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks></remarks>
                <System.Xml.Serialization.XmlElement("有効")> _
                Public Property Visible() As Boolean

                ''' <summary>
                ''' 前景色
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks>[Color]構造体はXML化できないので、別途[ForeColorCode]を用意する</remarks>
                <System.Xml.Serialization.XmlIgnoreAttribute()> _
                Public Property ForeColor() As Color

                ''' <summary>
                ''' 背景色
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks>[Color]構造体はXML化できないので、別途[BackColorCode]を用意する</remarks>
                <System.Xml.Serialization.XmlIgnoreAttribute()> _
                Public Property BackColor() As Color

                ''' <summary>
                ''' 前景色のコード
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks></remarks>
                <System.Xml.Serialization.XmlElement("前景色")> _
                Public Property ForeColorCode() As String
                    Get
                        Return Convert.ToString(ForeColor.ToArgb, 16)
                    End Get
                    Set(ByVal value As String)
                        ForeColor = Color.FromArgb(Convert.ToInt32(value, 16))
                    End Set
                End Property

                ''' <summary>
                ''' 背景色のコード
                ''' </summary>
                ''' <value></value>
                ''' <returns></returns>
                ''' <remarks></remarks>
                <System.Xml.Serialization.XmlElement("背景色")> _
                Public Property BackColorCode() As String
                    Get
                        Return Convert.ToString(BackColor.ToArgb, 16)
                    End Get
                    Set(ByVal value As String)
                        BackColor = Color.FromArgb(Convert.ToInt32(value, 16))
                    End Set
                End Property

            End Class

        End Class

    End Class





    '||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





    ''' <summary>
    ''' 情報パネルに関する設定項目
    ''' </summary>
    ''' <remarks></remarks>
    <Serializable()> _
    Public Class InformationPanelParameter

        ''' <summary>
        ''' 形式
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("形式")> _
        Public Property Style() As FormInformationPanel.LooksStyle

    End Class





    '||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





    ''' <summary>
    ''' 操作キーに関する設定項目
    ''' </summary>
    ''' <remarks></remarks>
    <Serializable()> _
    Public Class OperationKeyParameter

        ''' <summary>
        ''' コマンドパッド呼出キー
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks>ダイレクトに機能を利用するキーではないので、操作キーではあるけどダイレクトキーではない。</remarks>
        <System.Xml.Serialization.XmlElement("コマンドパッド呼出キー")> _
        Public Property CommandPadCallKey As Keys

        ''' <summary>
        ''' ダイレクトキーに関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks>操作キーの内のダイレクトキー</remarks>
        <System.Xml.Serialization.XmlElement("ダイレクトキー")> _
        Public Property DirectKey() As DirectKeyParameter = New DirectKeyParameter With {.CursorUpKey = Keys.Alt Or Keys.Up,
                                                                                         .CursorDownKey = Keys.Alt Or Keys.Down,
                                                                                         .CursorLeftKey = Keys.Alt Or Keys.Left,
                                                                                         .CursorRightKey = Keys.Alt Or Keys.Right,
                                                                                         .CopyKey = Keys.Shift Or Keys.Control Or Keys.C}

        ''' <summary>
        ''' シフトキーを押した時の不透明度を設定・取得します
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("Shift不透明度")> _
        Public Property ShiftOpacity As Double

        ''' <summary>
        ''' コントロールキーを押した時の不透明度を設定・取得します
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("Ctrl不透明度")> _
        Public Property ControlOpacity As Double



        '============================================================================================================================



        ''' <summary>
        ''' ダイレクトキーに関する設定項目
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()> _
        Public Class DirectKeyParameter

            ''' <summary>
            ''' カーソル上キー
            ''' </summary>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("カーソル上キー")> _
            Public Property CursorUpKey As Keys

            ''' <summary>
            ''' カーソル下キー
            ''' </summary>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("カーソル下キー")> _
            Public Property CursorDownKey As Keys

            ''' <summary>
            ''' カーソル左キー
            ''' </summary>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("カーソル左キー")> _
            Public Property CursorLeftKey As Keys

            ''' <summary>
            ''' カーソル右キー
            ''' </summary>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("カーソル右キー")> _
            Public Property CursorRightKey As Keys

            ''' <summary>
            ''' 拡大像複写キー
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("複写キー")> _
            Public Property CopyKey As Keys

        End Class

    End Class





    '||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





    ''' <summary>
    ''' コマンドパッドに関する設定項目
    ''' </summary>
    ''' <remarks></remarks>
    <Serializable()> _
    Public Class CommandPadParameter

        ''' <summary>
        ''' コマンドパッドの不透明度
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("不透明度")> _
        Public Property Opacity As Double = 0.5R

        ''' <summary>
        ''' 色書式に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("色書式")> _
        Public Property ColorFormat As ColorFormatParameter = New ColorFormatParameter With {.StandardFormat = New String() {"{3},{4},{5},{6}", "{3:X2},{4:X2},{5:X2},{6:X2}", "{1:X8}", "{2:X6}", "{6:X2}{5:X2}{4:X2}", "rgba({4}, {5}, {6}, {7:0.0})"},
                                                                                             .UseFormat = "{1:X8}"}



        '============================================================================================================================



        ''' <summary>
        ''' 色書式に関する設定項目
        ''' </summary>
        ''' <remarks></remarks>
        <Serializable()> _
        Public Class ColorFormatParameter

            ''' <summary>
            ''' 規定書式
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("規定書式")> _
            Public StandardFormat() As String

            ''' <summary>
            ''' 使用する書式
            ''' </summary>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("使用書式")> _
            Public UseFormat As String

        End Class

    End Class

End Class





'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





''' <summary>
''' 記録項目
''' </summary>
''' <remarks></remarks>
<System.Xml.Serialization.XmlRoot("HELuPERの記録内容")> _
Public Class ResumeParameter

    ''' <summary>
    ''' ビューアの表示位置
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>マウスからの回避モードが、[手動]の時に使われる</remarks>
    <System.Xml.Serialization.XmlElement("ビューアの表示位置")> _
    Public Property Locate() As Point = New Point(0, 0)

    ''' <summary>
    ''' 原点
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("原点")> _
    Public Property OriginPoint() As Point = New Point(0, 0)



    '============================================================================================================================



    Private memoLocate As Point '表示位置の記憶用
    Private memoOrigin As Point '原点の記憶用


    '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    ''' <summary>
    ''' 現在値を記憶しておく
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Memo()

        memoLocate = Locate
        memoOrigin = OriginPoint

    End Sub

    ''' <summary>
    ''' 表示位置または原点に変更があるか調べる
    ''' </summary>
    ''' <returns>変更があれば[True]、なければ[False]を返す</returns>
    ''' <remarks></remarks>
    Public Function Modified() As Boolean

        Return (memoLocate <> Locate OrElse memoOrigin <> OriginPoint)

    End Function

End Class