﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『画像の拡大について』
'  │・便利な標準機能の《Graphics.DrawImage》メソッドを使ってみたら、
'  │●問題点1・・・単純にこの機能を使って拡大しても、色の補間処理が行われるのでルーペにならない。《ScaleTransform》でワールド変換しても同じ結果。
'  │●問題点2・・・《Drawing2D.InterpolationMode.NearestNeighbor》指定で補間処理を無効化しても、本当に補間処理をすっ飛ばすだけで、補間に使用するスペースを空けた状態で拡大されてしまう。やっぱり、ルーペにならない。
'  │├→ しかたがないので、素直に“元画像から1ドットずつ色を読み取り、拡大先に四角を描画する”方法に落ち着いた。昔に比べればPCの能力は格段に向上しており、今回は拡大する範囲も限定的なので、この方法でも他の処理に大きな影響を与えずに済むと思う。
'  │└→ CPUに対して大した負荷ではないとはいえ、念のための更なる負荷軽減策として、拡大エリアと同じサイズの配列を用意してピクセル色を記憶しておき、前回と色が違う部分のみを描き直すようにしている。グラフィックメソッドより配列操作＆比較の方が処理が軽い場合に、簡単で有効な方法だと思う。
'  │
'  │『記録ファイル（"Resume.xml"）の保存タイミングについて』
'  │・記録ファイルは、アプリケーション終了時の状態を次回再開する時のために記録しておくファイルなので、単純に考えれば、アプリケーション終了時（すなわち《Form.Closing》イベント）に作成すれば良いということになる。
'  │・もちろん、本作を実行したまま、OSが終了する事態も考えられるけど、その時でもOSから各アプリケーションに終了要求が出されて《Form.Closing》イベントが発生するので問題ないと思っていた。
'  │・ところが、実際にOSを終了させてみると、なぜだかエラーが発生して正常終了できなくなる。
'  │→ そこで、別の経路（《Microsoft.Win32.SystemEvents.SessionEnding》イベント）でOSの終了を察知して、こちらでも処理できるようにしてみた。
'  │・この対策で、エラーは出なくなったけど、その原因は不明。イベントの発生タイミングに問題があるのか？ 例えば、《Form.Closing》イベント発生が遅すぎて、既にXML生成機能が終了してしまっているとか・・・。
'  │
'  │『マウスカーソルを回避するときの移動について』
'  │・拡大鏡が瞬間移動すると、目線移動が煩雑になって落ち着かない。しかも、注視している時に突然消えると集中力が途切れてしまう。
'  │　└→ これを解決するには、画面上をスムーズに移動させるしかない。
'  │・終始一定速度で移動する場合にスムーズに見えるようにするには、移動量を小さくしフレームレートを上げなければならない。しかし、フレームレートを上げるのは、プログラム（《Timer》コントロールの性能）的にもCPUへの負荷的にも不可能。
'  │　└→ そこで、加速することでこの問題を解決した。初速を遅くし徐々に加速することで、ユーザに「移動することとその方向」を意識させる。脳が移動していることを意識すると、移動量が大きくなって飛び飛びになっても違和感を感じ難くなる（その時、たまたまテレビでっ紹介されていたアニメーションのテクニックを参考にしてみた）。
'  │
'  │『フォーム表示位置の計算について』
'  │・拡大鏡のマウス回避処理や情報パネルの画面はみ出し回避処理で、各回避方法毎に同じような計算が記述されている。これらをまとめることも可能だけど、多種多様な回避方法を追加できるように、あえて独立性を高めている。
'  │
'  │『《Form.TransparencyKey》の透過設定について』
'  │・透明にすると、デスクトップの画像を取り込む際などに、そこにあるコントロールも含めてフォーム全体が無視され下の様子を拡大できる。また、透けて見えている物を操作することも可能となる。
'  │・しかし、実際に透過設定を行うと、いくつか問題が発生する。
'  │●問題点1・・・ビューアの《TransparencyKey》プロパティを設定した状態で、フォーム枠の有り無しを変更すると、デスクトップ画像が乱れる（タイトルバーの分ぐらいずれて復元される）現象が発生する。 
'  │└→ とりあえず、透過設定を一時的に解除してから枠を変更するようにすれば、回避できるみたい。
'  │●問題点2・・・MSDNの説明では、《TransparencyKey》プロパティに設定した色と同じ色のコントロールも透過すると書かれている。しかし、実際には、ほとんどの場合で透明にならない。コントロールが透明にならないのは逆に好都合なのだが、困ったことにごく稀に透明になることもあるみたい。再現性が低く、原因も対処法も不明。
'  │└→ この症状が出ると、拡大像に含まれる透過色と同じ色の部分が抜けてしまう。（拡大像をクリップボードにコピーしてペイントで見てみても透明になっていないので、この透過設定が原因だと考えられる。）
'  │・ビューアも、情報パネルの様に背景を透明にしたかったけど、このような不可解な動作をするため諦めた。
'  │
'  └────────────
'┌────┐
'│デザイン├──────────────────────
'└┬───┘
'  │『ダイレクトキーとコマンドパッドについて』
'  │・ダイレクトキーとコマンドパッドの2種類があるのは、ダイレクトでなくてもいい機能を一箇所に集めて、覚えないといけないキーの組み合わせを減らすため。
'  │・操作キー（ダイレクトキー含む）の定義は、機能を追加できるように、修飾子キーも含めて完全に独立させている。カーソルキーが、一つずつ定義されているのもそのため。
'  │
'  └────────────





''' <summary>
''' メインシステム 兼 ビューア
''' </summary>
''' <remarks>ビューアの振る舞いをしながらシステム全体を統括する。</remarks>
Public Class FormMain

    ''' <summary>キャンバス</summary>
    Private Structure Canvas

        ''' <summary>ビットマップ</summary>
        Dim Bmp As Bitmap

        ''' <summary>グラフィックス</summary>
        Dim Grp As Graphics

    End Structure


    '-----------------------------------------------------------------------------------------------------------


    Dim frmInfoPanel As FormInformationPanel       '情報パネルフォーム
    Dim WithEvents frmCommandPad As FormCommandPad 'コマンドパッドフォーム
    Dim frmAbout As FormAbout = Nothing            'バージョン情報フォーム
    Dim frmSetup As FormSetupConsole = Nothing     '設定コンソールフォーム

    Dim AreaSize As Size         '拡大範囲のサイズ
    Dim AreaCenter As Point      '拡大範囲の中心点
    Dim Magnification As Integer '拡大倍率

    Dim MouseLocate As Point = New Point(-1, -1) 'マウスカーソル座標

    Dim VirtualView(,) As Integer '仮想拡大像（現在と前回の拡大した全ピクセルの色コードを保持）

    Dim DesktopTranscript As Canvas 'デスクトップの複写用キャンバス
    Dim FoundationPart As Canvas    '下地部品
    Dim ViewScreen As Canvas        '拡大像
    Dim GridScreen As Canvas        '格子線
    Dim MarkerScreen As Canvas      'カーソルマーカー


    '-----------------------------------------------------------------------------------------------------------


    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        '=== Windowsの終了イベントを捕獲する ===
        AddHandler Microsoft.Win32.SystemEvents.SessionEnding, AddressOf SystemEvents_SessionEnding

        '=== 初期設定 ===
        '--- 設定ファイル読み込み ---
        Try
            Dim message As String = SettingLoad(SettingData)
            If message <> "" Then 'パラメータ異常修正
                MessageBox.Show("下記の重要な設定値に関して、設定ファイルの記述に異常がありました。" & vbCrLf & "修正値を適用して起動します。" & vbCrLf & vbCrLf & message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
            End If
        Catch ex As Exception
            MessageBox.Show("設定ファイルの読み込み・復元中に例外が発生しました。" & vbCrLf & "規定値を使用して起動を継続するので、設定をやり直してください。" & vbCrLf & vbCrLf & ex.Message,
                            Me.Text,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error) '読み込み中にエラーが発生した場合は、読み込めた部分も含め何も反映されないようなので、再初期化は不要。
        End Try


        '--- 記録ファイル読み込み ---
        Try
            ResumeLoad(ResumeData)
        Catch ex As Exception
            MessageBox.Show("記録ファイルの読み込み・復元中に例外が発生しました。" & vbCrLf & "規定値を使用して起動します。" & vbCrLf & vbCrLf & ex.Message,
                            Me.Text,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error) '読み込み中にエラーが発生した場合は、読み込めた部分も含め何も反映されないようなので、再初期化は不要。
        End Try
        ResumeData.Memo() '終了時に変更を判断するため、起動時の値を記憶しておく。

        '--- コマンドパッド準備 ---
        frmCommandPad = New FormCommandPad With {.Owner = Me}

        '--- 情報パネル準備 ---
        frmInfoPanel = New FormInformationPanel With {.Owner = Me}

        '--- ビューア準備 ---
        MakeViewer()

    End Sub

    Private Sub FormMain_Shown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shown

        '=== 情報パネルを表示 ===
        frmInfoPanel.StyleChange()

        '=== タスクトレイにアイコンを登録 ===
        NotifyIcon1.Icon = My.Resources.LupeOn
        NotifyIcon1.Visible = True

        '=== 各種タイマ始動 ===
        tmrRefresh.Enabled = True
        tmrAvoidTraveler.Enabled = True
        tmrOpeKeyScan.Enabled = True

    End Sub

    Private Sub SystemEvents_SessionEnding(ByVal sender As Object, ByVal e As Microsoft.Win32.SessionEndingEventArgs)

        '=== 常駐しながらWindowsが終了した場合の記録保存 ===
        ResumeSaveing()

    End Sub

    Private Sub FormMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

        '=== 通常終了時の記録保存 ===
        ResumeSaveing()

    End Sub

    Private Sub FormMain_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed

        '=== Windowsの終了イベント捕捉を解除する ===
        RemoveHandler Microsoft.Win32.SystemEvents.SessionEnding, AddressOf SystemEvents_SessionEnding

        '=== タスクトレイからアイコン消去 ===
        NotifyIcon1.Visible = False

    End Sub

    Private Sub FormMain_Move(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Move, Me.Resize

        '=== 情報パネルを移動させる ===
        If frmInfoPanel IsNot Nothing Then
            frmInfoPanel.MoveLocate()
        End If

    End Sub

    Private Sub FormMain_ResizeEnd(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.ResizeEnd
        'この[ResizeEnd]イベントは、ユーザがフォームを移動し終えた時にも発生する（コードでプロパティを変更しても発生しない）ので、“手動モード時の移動終了”イベントとして利用している。

        '=== 手動モードの場合は、ビューアの表示位置を記録する ===
        If SettingData.Lupe.Avoid.Mode = AvoidType.Passive Then
            ResumeData.Locate = Me.Location
        End If

    End Sub

    Private Sub NotifyIcon1_MouseClick(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles NotifyIcon1.MouseClick

        '=== 表示/非表示の切り替え ===
        If e.Button = Windows.Forms.MouseButtons.Left Then
            tsmHide.PerformClick()
        End If

    End Sub

    Private Sub ContextMenuStrip1_Opening(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles ContextMenuStrip1.Opening

        '=== 子フォームの表示中は、コンテクストメニューを開けない ===
        Dim openSubForm As Form = Nothing
        If frmAbout IsNot Nothing Then
            openSubForm = frmAbout
        ElseIf frmSetup IsNot Nothing Then
            openSubForm = frmSetup
        End If

        If openSubForm IsNot Nothing Then
            e.Cancel = True
            NotifyIcon1.ShowBalloonTip(5000, String.Format("『{0}』が、開いたままになっています", openSubForm.Text), "いったん閉じてから、作業を続けてください。", ToolTipIcon.Info)

            openSubForm.Activate()
        End If

    End Sub

    Private Sub tsmQuit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmQuit.Click

        '=== 終了する ===
        Me.Close()

    End Sub

    Private Sub tsmAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmAbout.Click

        '=== バージョン情報を表示する ===
        frmAbout = New FormAbout
        frmAbout.ShowDialog(Me)
        frmAbout = Nothing

    End Sub

    Private Sub tsmSetting_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmSetting.Click

        '=== 設定 ===
        Me.TopMost = False '最上位の状態で設定コンソールを表示すると、（操作キー一覧等の）ToolTipがフォームの裏側に表示されてしまい読めない不具合が発生するため、一時的に解除する。

        frmSetup = New FormSetupConsole
        If frmSetup.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
            AllNew()
        End If
        frmSetup = Nothing

        Me.TopMost = True

    End Sub

    Private Sub tsmHide_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles tsmHide.Click

        '=== 表示/非表示の切り替え ===
        '--- ビューア ---
        Me.Visible = Not DirectCast(sender, ToolStripMenuItem).Checked

        '--- 情報パネルの表示/非表示 ---
        If SettingData.Lupe.InformationPanel.Style <> FormInformationPanel.LooksStyle.Unvisible Then
            frmInfoPanel.Visible = Me.Visible
        End If

        '--- タスクトレイのアイコンを変更 ---
        NotifyIcon1.Icon = IIf(Me.Visible, My.Resources.LupeOn, My.Resources.LupeOff)

    End Sub

    Private Sub tmrRefresh_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrRefresh.Tick

#If DEBUG Then

        '=== 拡大鏡更新（デバッグ用） ===
        If SettingData.Lupe.Avoid.Mode = AvoidType.Passive Then '手動時には、更新に掛かった時間を計測する。
            Dim sw As New Stopwatch
            sw.Start()
            RefreshLupe()
            sw.Stop()
            Me.Text = sw.ElapsedMilliseconds & "ms"
        Else
            RefreshLupe()
        End If

#Else

        '=== 拡大鏡更新（リリース用） ===
        RefreshLupe()

#End If

    End Sub

    Private Sub picViewer_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs) Handles picViewer.Paint

        '=== コマンドパッド使用中は砂時計表示 ===
        If frmCommandPad.Visible Then
            Cursors.WaitCursor.Draw(e.Graphics, New Rectangle(4, 4, picViewer.ClientSize.Width, picViewer.ClientSize.Height))
        End If

    End Sub

    Private Sub FrmCommandPad_Action(sender As Object, e As CommandEventArgs) Handles frmCommandPad.Action

        Select Case e.Request
            Case CommandEventArgs.Requests.OriginSet

                '=== 原点をマウスカーソル位置に設定する ===
                ResumeData.OriginPoint = MouseLocate

            Case CommandEventArgs.Requests.OriginReset

                '=== 原点を(0,0)に戻す ===
                ResumeData.OriginPoint = New Point(0, 0)

            Case CommandEventArgs.Requests.GotoOrigin

                '=== マウスカーソルを原点へ移動する ===
                Windows.Forms.Cursor.Position = ResumeData.OriginPoint
                MouseLocate = ResumeData.OriginPoint '拡大鏡も更新する。

            Case CommandEventArgs.Requests.GetColor

                '=== 色コードをクリップボードへコピーする ===
                Try
                    Clipboard.SetText(GetColorString(GetMousePointColor, SettingData.CommandPad.ColorFormat.UseFormat))
                Catch ex As Exception
                    MessageBox.Show("色書式が異常です。", Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
                    Exit Sub
                End Try

            Case CommandEventArgs.Requests.CopyView

                '=== 拡大像をクリップボードへコピーする ===
                Clipboard.SetImage(ViewScreen.Bmp)

            Case Else

                Exit Sub

        End Select

        '--- 完了音 ---
        Beep()
        'System.Media.SystemSounds.Asterisk.Play()

    End Sub

    ''' <summary>
    ''' 新調する
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub AllNew()

        '=== 拡大鏡の刷新 ===
        MakeViewer()
        DrawView()
        frmInfoPanel.StyleChange()

    End Sub

    ''' <summary>
    ''' 設定内容に合わせてビューアを作る
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub MakeViewer()

        With SettingData.Lupe

            '=== 描画パラメータ作成 ===
            With .View
                With .Area
                    AreaSize = New Size(.StandardSize(.SelectWidthSizeIndex), .StandardSize(.SelectHeightSizeIndex))
                End With
                AreaCenter = New Point(AreaSize.Width \ 2, AreaSize.Height \ 2)
                Magnification = .Magnification.StandardChoices(.Magnification.SelectIndex)
            End With

            '--- 仮想拡大像の初期化 ---
            ReDim VirtualView(AreaSize.Width, AreaSize.Height)
            For y As Integer = 0 To AreaSize.Height - 1
                For x As Integer = 0 To AreaSize.Width - 1
                    VirtualView(x, y) = &H1000000 '有り得なさそうな値で埋めておく。（もしくは、拡大像を下地で埋めておく方法もアリ。）
                Next
            Next

            '=== フォーム設定 ===
            '--- 外観 ---
            Select Case .Avoid.Mode
                Case AvoidType.Passive
                    Me.FormBorderStyle = Windows.Forms.FormBorderStyle.FixedToolWindow
                Case Else
                    Me.FormBorderStyle = Windows.Forms.FormBorderStyle.None
            End Select

            '--- サイズ ---
            Dim gridSpace As Integer = IIf(.View.Draw.Grid.Visible, 1, 0) '右端と下端の格子線を表示するスペース。
            Me.Size += New Size((AreaSize.Width * Magnification + gridSpace) - picViewer.ClientSize.Width, (AreaSize.Height * Magnification + gridSpace) - picViewer.ClientSize.Height)

            '--- 回避の初期化 ---
            AvoidInitialize()

            '=== 更新間隔 ===
            tmrRefresh.Interval = .RefreshInterval

        End With

        '=== 各キャンバス設定 ===
        '--- デスクトップ画像取り込み用 ---
        With DesktopTranscript
            .Bmp = New Bitmap(AreaSize.Width, AreaSize.Height)
            .Grp = Graphics.FromImage(.Bmp)
        End With

        '--- 下地部品用 ---
        With FoundationPart
            .Bmp = New Bitmap(Magnification, Magnification)
            .Grp = Graphics.FromImage(.Bmp)

            .Grp.Clear(SettingData.Lupe.View.Draw.Foundation.BackColor)

            If SettingData.Lupe.View.Draw.Foundation.Visible Then
                .Grp.DrawLine(New Pen(SettingData.Lupe.View.Draw.Foundation.ForeColor), 0, 0, .Bmp.Width, .Bmp.Height)
            End If
        End With

        '--- 拡大像用 ---
        With ViewScreen
            .Bmp = New Bitmap(picViewer.ClientSize.Width, picViewer.ClientSize.Height)
            .Grp = Graphics.FromImage(.Bmp)
            picViewer.Image = .Bmp
        End With

        '--- 格子線用 ---
        With GridScreen
            .Bmp = ViewScreen.Bmp.Clone
            .Grp = Graphics.FromImage(.Bmp)

            .Grp.Clear(Color.Transparent)

            Dim p As Pen = New Pen(SettingData.Lupe.View.Draw.Grid.ForeColor)
            For i As Integer = 0 To .Bmp.Width Step Magnification
                .Grp.DrawLine(p, i, 0, i, .Bmp.Height)
            Next
            For i As Integer = 0 To .Bmp.Height Step Magnification
                .Grp.DrawLine(p, 0, i, .Bmp.Width, i)
            Next
        End With

        '--- カーソルマーカー用 ---
        With MarkerScreen
            .Bmp = ViewScreen.Bmp.Clone
            .Grp = Graphics.FromImage(.Bmp)

            .Grp.Clear(Color.Transparent)

            Dim rect As Rectangle = New Rectangle(AreaCenter.X * Magnification, AreaCenter.Y * Magnification, Magnification, Magnification)
            .Grp.DrawRectangle(New Pen(SettingData.Lupe.View.Draw.CursorMarker.ForeColor), rect)
            rect.Inflate(1, 1)
            .Grp.DrawRectangle(New Pen(SettingData.Lupe.View.Draw.CursorMarker.BackColor), rect)
        End With

    End Sub

    ''' <summary>
    ''' 拡大鏡を更新する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RefreshLupe()

        '=== マウスカーソル位置の更新 ===
        If Not frmCommandPad.Visible Then 'コマンドパッド使用中は、マウスに追従しない。
            MouseLocate = System.Windows.Forms.Cursor.Position
        End If

        '=== 拡大鏡描画 ===
        DrawView()

        '=== 情報パネル更新 ===
        frmInfoPanel.WriteParameter(MouseLocate, GetMousePointColor)

    End Sub

    ''' <summary>
    ''' 拡大像を描画する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DrawView()

        '=== デスクトップを取り込む ===
        With DesktopTranscript
            .Grp.Clear(Color.Transparent)
            .Grp.CopyFromScreen(MouseLocate - AreaCenter, New Point(0, 0), AreaSize) '←この1行だけで画面を取り込めちゃう！！
        End With

        With ViewScreen.Grp

            '=== 拡大像 ===
            Dim pixelColor As Color
            Dim redrawFlag As Boolean = False '描き直し済みフラグ

            For y As Integer = 0 To AreaSize.Height - 1
                For x As Integer = 0 To AreaSize.Width - 1

                    pixelColor = DesktopTranscript.Bmp.GetPixel(x, y) '色取得

                    If VirtualView(x, y) <> pixelColor.ToArgb Then '前回の色と違っていたら・・・

                        If pixelColor.A < &HFF Then
                            .DrawImage(FoundationPart.Bmp, x * Magnification, y * Magnification) '透明色用下地部品描画
                        End If

                        .FillRectangle(New SolidBrush(pixelColor), x * Magnification, y * Magnification, Magnification, Magnification) 'ピクセル描画
                        '画面の領域外にしか透明色は存在しないという前提なら、透明色をわざわざ描画する必要はないけど、貧しい知識の外側で何かあったときのために一応全てのピクセルを描いておく。

                        VirtualView(x, y) = pixelColor.ToArgb '仮想拡大像に色を記録
                        redrawFlag = True

                    End If

                Next
            Next

            '=== 補助要素 ===
            If redrawFlag Then

                '--- 格子線 ---
                If SettingData.Lupe.View.Draw.Grid.Visible Then
                    .DrawImage(GridScreen.Bmp, 0, 0)
                End If

                '--- カーソルマーカー ---
                If SettingData.Lupe.View.Draw.CursorMarker.Visible Then
                    .DrawImage(MarkerScreen.Bmp, 0, 0)
                End If

            End If

        End With

        picViewer.Refresh()

    End Sub

    ''' <summary>
    ''' マウスカーソル位置の色を取得する
    ''' </summary>
    ''' <returns>色コード(ARGB値)</returns>
    ''' <remarks></remarks>
    Private Function GetMousePointColor() As Integer

        Return VirtualView(AreaCenter.X, AreaCenter.Y)

    End Function

    ''' <summary>
    ''' 色コードをARGBの各要素に分離する
    ''' </summary>
    ''' <param name="ColorCode">色コード(ARGB値)</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function SplitARGB(ColorCode As Integer) As Byte()

        '※情報パネルからも利用される。

        Dim argb(3) As Byte
        argb(0) = ColorCode >> 24 And &HFF 'アルファ要素の取り出し：カラーコードを24bit分右シフトして、下1Byte分を取り出す。
        argb(1) = ColorCode >> 16 And &HFF '赤要素の取り出し      ：カラーコードを16bit分右シフトして、下1Byte分を取り出す。
        argb(2) = ColorCode >> 8 And &HFF  '緑要素の取り出し      ：カラーコードを8bit分右シフトして、下1Byte分を取り出す。
        argb(3) = ColorCode And &HFF       '青要素の取り出し      ：カラーコードの下1Byte分を取り出す。

        Return argb

    End Function

    ''' <summary>
    ''' 色コードを色書式で表現する
    ''' </summary>
    ''' <param name="ColorCode">色コード(ARGB値)</param>
    ''' <param name="Format">色書式</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetColorString(ColorCode As Integer, Format As String) As String

        '※設定コンソールからも利用される。

        Dim rgb As Integer = ColorCode And &HFFFFFF
        Dim argb() As Byte = SplitARGB(ColorCode)

        Try
            If Format <> "" Then
                Return String.Format(Format, vbCrLf, ColorCode, rgb, argb(0), argb(1), argb(2), argb(3), CSng(argb(0) / 255))
            Else
                Return ColorCode.ToString("X8") '書式が未記入の場合の臨時書式。
            End If
        Catch ex As Exception
            Throw
        End Try

    End Function

    ''' <summary>
    ''' レジュームを保存する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ResumeSaveing()

        Static ResumeSaveComplete As Boolean = False 'レジューム保存済フラグ

        If Not ResumeSaveComplete Then

            '=== 起動時に対して変更があれば保存する ===
            If ResumeData.Modified Then
                Try
                    ResumeSave()
                Catch ex As Exception
                    MessageBox.Show("記録ファイルの保存中に例外が発生しました。" & vbCrLf & vbCrLf & ex.Message,
                                    Me.Text,
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Error)
                End Try
            End If

            ResumeSaveComplete = True

        End If

    End Sub

End Class
