﻿'[FormMain]クラスの続き
'マウス回避に係わるルーチンをまとまる。

Partial Class FormMain

    ''' <summary>
    ''' マウスカーソルからの回避方法
    ''' </summary>
    ''' <remarks></remarks>
    Public Enum AvoidType As Byte

        ''' <summary>手動</summary>
        ''' <remarks>通常のウィンドウと同じ</remarks>
        Passive

        ''' <summary>画面の中央を左右</summary>
        ScreenSide

        ''' <summary>画面の右の3地点を行ったり来たり</summary>
        RightTriangle

        ''' <summary>画面の四隅</summary>
        ScreenCorner

        ''' <summary>マウスカーソルと一緒</summary>
        Randezvous

    End Enum


    '-----------------------------------------------------------------------------------------------------------


    ''' <summary>
    ''' 回避パターン
    ''' </summary>
    ''' <remarks></remarks>
    Public Structure AvoidPattern

        ''' <summary>呼称</summary>
        Public Property Caption As String

        ''' <summary>回避法</summary>
        Public Property Type As AvoidType

    End Structure


    '-----------------------------------------------------------------------------------------------------------


    ''' <summary>
    ''' 用意されたマウスカーソルからの回避方法のコレクション
    ''' </summary>
    ''' <remarks>設定コンソールの[ComboBox]用</remarks>
    Public ReadOnly PreparedAvoid As List(Of AvoidPattern) = {New AvoidPattern With {.Caption = "手取り足取り", .Type = AvoidType.Passive},
                                                              New AvoidPattern With {.Caption = "画面中央の右往左往", .Type = AvoidType.ScreenSide},
                                                              New AvoidPattern With {.Caption = "画面右の三角関係", .Type = AvoidType.RightTriangle},
                                                              New AvoidPattern With {.Caption = "画面四隅のすれ違い", .Type = AvoidType.ScreenCorner},
                                                              New AvoidPattern With {.Caption = "画面の中の金魚", .Type = AvoidType.Randezvous}}.ToList


    '-----------------------------------------------------------------------------------------------------------


    ''' <summary>初速</summary>
    Private Const INITIAL_SPEED As Integer = 4

    ''' <summary>加速度</summary>
    Private Const ACCELERATION As Integer = 4



    '===========================================================================================================



    ''' <summary>回避先</summary>
    Dim AvoidDestination As Point

    ''' <summary>回避速度</summary>
    Dim AvoidSpeed As Integer

    ''' <summary>回避先到着フラグ</summary>
    Dim AvoidGoal As Boolean = True



    '===========================================================================================================



    Private Sub tmrAvoidTraveler_Tick(sender As System.Object, e As System.EventArgs) Handles tmrAvoidTraveler.Tick

        '=== 回避旅行 ===
        AvoidPlan()

        If Not AvoidGoal Then
            AvoidMove()
        End If

    End Sub



    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::



    ''' <summary>
    ''' 回避の初期化
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub AvoidInitialize()

        '=== とりあえずの表示位置 ===
        With Screen.GetWorkingArea(Me)

            Select Case SettingData.Lupe.Avoid.Mode
                Case AvoidType.Passive
                    Me.Location = ResumeData.Locate
                Case AvoidType.ScreenSide
                    AvoidDestination = New Point(.Right - Me.Width, .Top + (.Height - Me.Height) \ 2) '右中央
                Case AvoidType.ScreenCorner
                    AvoidDestination = New Point(.Right - Me.Width, .Top) '右上
                Case AvoidType.Randezvous
                    AvoidPlanRandezvous(True)
            End Select

        End With

    End Sub

    ''' <summary>
    ''' マウスカーソルからの回避計画を練る
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub AvoidPlan()

        With SettingData.Lupe.Avoid

            If .Mode <> AvoidType.Passive Then

                '=== 新しい表示位置の計算 ===
                Dim newLocate As Point '新しい表示位置

                Select Case .Mode

                    Case AvoidType.ScreenSide

                        '--- 左右 ---
                        newLocate = AvoidPlanScreenSide()

                    Case AvoidType.RightTriangle

                        '--- 三角 ---
                        newLocate = AvoidPlanTriangle()

                    Case AvoidType.ScreenCorner

                        '--- 四隅 ---
                        newLocate = AvoidPlanScreenCorner()

                    Case AvoidType.Randezvous

                        '--- 帯同 ---
                        newLocate = AvoidPlanRandezvous()

                End Select

                '=== 新しい表示位置を決定 ===
                If Me.Location <> newLocate Then

                    AvoidDestination = newLocate

                    If AvoidGoal Then
                        AvoidGoal = False
                        AvoidSpeed = INITIAL_SPEED '初速
                        'Beep()
                    End If

                End If

            End If

        End With

    End Sub

    ''' <summary>
    ''' マウス回避：『画面中央の右往左往』
    ''' </summary>
    ''' <returns>新しい移動先</returns>
    ''' <remarks></remarks>
    Private Function AvoidPlanScreenSide() As Point

        Dim result As Point '移動先
        Dim horizonMargin As Integer '実際の水平方向のマージン

        '=== 必要なマージン ===
        If frmInfoPanel.Visible AndAlso Me.Width < frmInfoPanel.Width Then '情報パネルの横幅がビューアより大きい場合を考慮する。
            horizonMargin = frmInfoPanel.Width + SettingData.Lupe.Avoid.Margin
        Else
            horizonMargin = Me.Width + SettingData.Lupe.Avoid.Margin
        End If

        '=== 移動先 ===
        With Screen.GetWorkingArea(Me)

            If Windows.Forms.Cursor.Position.X < .Left + horizonMargin Then
                result.X = .Right - Me.Width '画面右側へ
            ElseIf Windows.Forms.Cursor.Position.X > .Right - horizonMargin Then
                result.X = .Left '画面左側へ
            Else
                Return AvoidDestination '現状維持
            End If

            result.Y = AvoidDestination.Y '現状維持（画面中央固定）
            Return result

        End With

    End Function

    ''' <summary>
    ''' マウス回避：『画面右の三角関係』
    ''' </summary>
    ''' <returns>新しい移動先</returns>
    ''' <remarks></remarks>
    Private Function AvoidPlanTriangle() As Point

        Dim result As Point '移動先
        Dim borderLine As Integer '回避行動を行う境界線
        Dim eden As Point '安寧の地

        With Screen.GetWorkingArea(Me)

            '=== 境界線 ===
            If frmInfoPanel.Visible AndAlso Me.Width < frmInfoPanel.Width Then '情報パネルの横幅がビューアより大きい場合を考慮する。
                borderLine = .Right - frmInfoPanel.Width - SettingData.Lupe.Avoid.Margin
            Else
                borderLine = .Right - Me.Width - SettingData.Lupe.Avoid.Margin
            End If

            '=== 移動先 ===
            eden = New Point(.Right - Me.Width, .Top + (.Height - Me.Height) \ 2)

            If Windows.Forms.Cursor.Position.X > borderLine Then

                '--- 画面の上下中央へ出張 ---
                If Me.Location = eden Then

                    '初出張の場合・・・マウスカーソルが、上半分にあるか？ 下半分にあるか？
                    If Windows.Forms.Cursor.Position.Y > .Top + .Height \ 2 Then
                        result.Y = .Top '上へ
                    Else
                        result.Y = .Bottom - Me.Height '下へ
                    End If

                Else

                    '出張中の場合・・・マウスカーソルが、ビューアの許容範囲内に入ったか？
                    If Windows.Forms.Cursor.Position.Y > .Height - Me.Height - SettingData.Lupe.Avoid.Margin Then
                        result.Y = .Top '上へ
                    ElseIf Windows.Forms.Cursor.Position.Y < Me.Height + SettingData.Lupe.Avoid.Margin Then
                        result.Y = .Bottom - Me.Height '下へ
                    Else
                        Return AvoidDestination '現状維持
                    End If

                End If

                result.X = .Left + (.Width - Me.Width) \ 2 '画面中央
                Return result

            Else

                '--- 定位置（画面右中央）で安寧 ---
                Return eden

            End If

        End With

    End Function

    ''' <summary>
    ''' マウス回避：『画面四隅のすれ違い』
    ''' </summary>
    ''' <returns>新しい移動先</returns>
    ''' <remarks></remarks>
    Private Function AvoidPlanScreenCorner() As Point

        Dim result As Point '移動先

        Dim horizonMargin As Integer = Me.Width + SettingData.Lupe.Avoid.Margin '実際の右側マージン
        Dim verticalMargin As Integer = Me.Height + SettingData.Lupe.Avoid.Margin '実際の下側マージン

        '=== 移動先 ===
        With Screen.GetWorkingArea(Me)
            If Windows.Forms.Cursor.Position.X < .Left + horizonMargin Then
                result.X = .Right - Me.Width '画面右端へ
            ElseIf Windows.Forms.Cursor.Position.X > .Right - horizonMargin Then
                result.X = .Left '画面左端へ
            Else
                result.X = AvoidDestination.X '現状維持
            End If

            If Windows.Forms.Cursor.Position.Y < .Top + verticalMargin Then
                result.Y = .Bottom - Me.Height '画面下端へ
            ElseIf Windows.Forms.Cursor.Position.Y > .Bottom - verticalMargin Then
                result.Y = .Top '画面上端へ
            Else
                result.Y = AvoidDestination.Y '現状維持
            End If
        End With

        Return result

    End Function

    ''' <summary>
    ''' マウス回避：『画面の中の金魚』
    ''' </summary>
    ''' <param name="Initialize">初期化フラグ</param>
    ''' <returns>新しい移動先</returns>
    ''' <remarks></remarks>
    Private Function AvoidPlanRandezvous(Optional Initialize As Boolean = False) As Point

        '画面端に近づき、マウスカーソルに対する位置取りが変わった時、移動速度が速いままだと一気に飛んでしまうので違和感がある。
        '└→ そこで、位置取りに変化が発生したら、一旦ゴールしたことにして、初速を元に戻させるようにしている。

        Const leftTop As Integer = -1 '左または上側のポジション
        Const rightBottom As Integer = 1 '右または下側のポジション

        Static horizonPositioning As Integer = 0 '現在のマウスカーソルに対する水平方向での位置取り
        Static verticalPositioning As Integer = 0 '現在のマウスカーソルに対する垂直方向での位置取り

        Dim result As Point '移動先

        Dim horizonMargin As Integer = Me.Width + SettingData.Lupe.Avoid.Margin '実際の右側マージン
        Dim verticalMargin As Integer = Me.Height + SettingData.Lupe.Avoid.Margin '実際の下側マージン

        '=== 初期化 ===
        If Initialize Then
            horizonPositioning = 0
            verticalPositioning = 0
        End If

        '=== 移動先 ===
        With Screen.GetWorkingArea(Me)

            If Windows.Forms.Cursor.Position.X > .Right - horizonMargin Then

                result.X = Windows.Forms.Cursor.Position.X - horizonMargin '左側へ

                If horizonPositioning <> leftTop Then
                    horizonPositioning = leftTop
                    AvoidGoal = True '水平方向の位置取りが変わったら、初速を戻す（一旦ゴールしたことにする）。
                End If

            Else

                result.X = Windows.Forms.Cursor.Position.X + SettingData.Lupe.Avoid.Margin '右側へ

                If horizonPositioning <> rightBottom Then
                    horizonPositioning = rightBottom
                    AvoidGoal = True '垂直方向の位置取りが変わったら、初速を戻す（一旦ゴールしたことにする）。
                End If

            End If

            If Windows.Forms.Cursor.Position.Y > .Bottom - verticalMargin Then

                result.Y = Windows.Forms.Cursor.Position.Y - verticalMargin '上側へ

                If verticalPositioning <> leftTop Then
                    verticalPositioning = leftTop
                    AvoidGoal = True '水平方向の位置取りが変わったら、初速を戻す（一旦ゴールしたことにする）。
                End If

            Else

                result.Y = Windows.Forms.Cursor.Position.Y + SettingData.Lupe.Avoid.Margin '下側へ

                If verticalPositioning <> rightBottom Then
                    verticalPositioning = rightBottom
                    AvoidGoal = True '垂直方向の位置取りが変わったら、初速を戻す（一旦ゴールしたことにする）。
                End If

            End If

        End With

        Return result

    End Function

    ''' <summary>
    ''' 回避行動
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub AvoidMove()

        '=== 移動 ===
        Dim x, y As Integer

        x = AvoidDestination.X - Me.Left
        y = AvoidDestination.Y - Me.Top

        If x <> 0 OrElse y <> 0 Then

            If AvoidSpeed < 1000 Then AvoidSpeed *= ACCELERATION '加速！（1000は念のための規制）

            If Math.Abs(x) > AvoidSpeed Then x = AvoidSpeed * Math.Sign(x)
            If Math.Abs(y) > AvoidSpeed Then y = AvoidSpeed * Math.Sign(y)

            Me.Left += x
            Me.Top += y

        Else

            '--- 回避先到着 ---
            AvoidGoal = True

        End If

    End Sub

End Class
