﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『キーの押下とAPI《GetAsyncKeyState》の返り値の遷移』
'  │                                 ┌一発目    ┌中休み        ┌リピート有効      ┌名残り（次回APIが呼ばれるまで残る）
'  │ 何れかのキーを・・・│押す前 → │押した → │押し続けた → │更に押し続けた → │放した →→→→→→→→→→→→→→→ │押していない
'  │    APIの返り値・・・│&H0000 → │&H8001 → │&H8000 →→→ │&H8001 →→→→→ │&H0001 →→→→→→→→→→→→→→→ │&H0000
'  │《PressStatus》・・・└Off       └Valid     └On            └Valid             └last                                  └Off
'  │※それなりに高頻度にスキャンしないと、キーリピート機能を有効に検知できない。
'  └────────────





'[FormMain]クラスの続き
'操作キーに係わるルーチンをまとまる。

Partial Class FormMain

    ''' <summary>
    ''' [API] キーボード スキャン
    ''' </summary>
    ''' <param name="vKey">調べたいキーのコード</param>
    ''' <returns>現在押されていたら最上位bit(16bit目)が立つ。前回の呼び出し以降に押されていたら最下位bitが立つ（こちらは、キーリピート機能に対応）。押されていなかったら0が返される。</returns>
    ''' <remarks></remarks>
    Declare Function GetAsyncKeyState Lib "user32.dll" (ByVal vKey As Integer) As Short


    '-----------------------------------------------------------------------------------------------------------


    ''' <summary>
    ''' 押下状態
    ''' </summary>
    ''' <remarks></remarks>
    Public Enum PressStatus As Short

        ''' <summary>押されていない</summary>
        Off = &H0S

        ''' <summary>押されている</summary>
        [On] = &H8000S

        ''' <summary>有効（キーリピート機能）</summary>
        Valid = &H8001S

        ''' <summary>押されていた</summary>
        last = &H1S

    End Enum



    '===========================================================================================================



    Private Sub tmrOpeKeyScan_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrOpeKeyScan.Tick

        '=== 操作キー ===
        CallKey()
        DirectKey()

        '=== 一時透過 ===
        '複数のキーを同時に押している時は、透過しない。

        Dim opacityValue As Double '不透明度

        If My.Computer.Keyboard.ShiftKeyDown AndAlso Not My.Computer.Keyboard.CtrlKeyDown AndAlso Not My.Computer.Keyboard.AltKeyDown Then
            opacityValue = SettingData.OperationKey.ShiftOpacity
        ElseIf My.Computer.Keyboard.CtrlKeyDown AndAlso Not My.Computer.Keyboard.ShiftKeyDown AndAlso Not My.Computer.Keyboard.AltKeyDown Then
            opacityValue = SettingData.OperationKey.ControlOpacity
        Else
            opacityValue = 1.0R
        End If

        If Me.Opacity <> opacityValue Then
            Me.Opacity = opacityValue
            frmInfoPanel.Opacity = opacityValue
        End If

    End Sub



    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::



    ''' <summary>
    ''' コマンドパッド呼出キー
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub CallKey()

        With frmCommandPad

            If (KeyScan(SettingData.OperationKey.CommandPadCallKey) And PressStatus.On) = PressStatus.On Then

                '=== 出現 ===
                If Not .Visible Then

                    '表示位置計算
                    .Location = Windows.Forms.Cursor.Position - .btnOriginSet.Location - New Size(.btnOriginSet.Width \ 2, .btnOriginSet.Height \ 2) '基本は原点設定ボタンの中心

                    '画面からのはみ出し補正
                    Dim myScreen As Rectangle = Screen.GetBounds(Me)
                    If .Top < myScreen.Top Then
                        .Top = myScreen.Top
                    ElseIf .Top > myScreen.Bottom - .Height Then
                        .Top = myScreen.Bottom - .Height
                    End If
                    If .Left < myScreen.Left Then
                        .Left = myScreen.Left
                    ElseIf .Left > myScreen.Right - .Width Then
                        .Left = myScreen.Right - .Width
                    End If

                    .Visible = True

                End If

            Else

                '=== 消失 ===
                If .Visible Then
                    .Visible = False
                End If

            End If

        End With

    End Sub

    ''' <summary>
    ''' ダイレクトキーの操作
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub DirectKey()

        With SettingData.OperationKey.DirectKey

            '=== 拡大像をクリップボードへコピーする ===
            'コマンドパッドに同機能を設けたから、おまけ。
            If KeyScan(.CopyKey) = PressStatus.Valid Then
                FrmCommandPad_Action(Me, New CommandEventArgs(CommandEventArgs.Requests.CopyView)) 'コマンドパッドのイベントを拝借する。
                Exit Sub '※※※ダイレクトキーは、一度に一機能しか使用しない。
            End If

            '=== マウスカーソル移動 ===
            '・スキャンレートが低い場合でも押下に確実に反応するように、最下位bit[PressStatus.last]のみをチェックしている。副作用として、キー入力を終了しても次にスキャンするまで立ったままなので、タイミングによっては1回余分に作動してしまう。
            '  └→ 言ってしまえば、スキャンレートを低くしないなら意味がない。
            '・API[GetAsyncKeyState]の最下位bitを利用してキーリピート機能（1回目と2回目の間に間隔を空ける）を実現する場合、カーソルキーの斜め入力への完全な対応が難しくなる。
            '  └→ 2箇所のキーを同時に押した後、後から押したキーを放すともう一方のキーの最下位bitが立っていないので、押されている状態を有効だと判断できないため。

            Dim x, y As Integer

            '--- 左右 ---
            If (KeyScan(.CursorLeftKey) And PressStatus.last) = PressStatus.last Then
                x = -1
            ElseIf (KeyScan(.CursorRightKey) And PressStatus.last) = PressStatus.last Then
                x = +1
            End If

            '--- 上下 ---
            If (KeyScan(.CursorUpKey) And PressStatus.last) = PressStatus.last Then
                y = -1
            ElseIf (KeyScan(.CursorDownKey) And PressStatus.last) = PressStatus.last Then
                y = +1
            End If

            '--- 移動 ---
            If x <> 0 OrElse y <> 0 Then
                Windows.Forms.Cursor.Position += New Point(x, y)
            End If

        End With

    End Sub

    ''' <summary>
    ''' キーが押されているかを調べる
    ''' </summary>
    ''' <param name="TargetKeySet">調べるキーセット</param>
    ''' <returns>押下状態</returns>
    ''' <remarks></remarks>
    Public Function KeyScan(TargetKeySet As Keys) As PressStatus

        Dim state As Keys = Keys.None '押下状態
        Dim apiReturn As PressStatus '[GetAsyncKeyState]APIの戻り値

        Dim pureKey As Keys = TargetKeySet And Keys.KeyCode 'キーセットから純粋なキーを抽出する（修飾子キーを取り除く）。
        Dim scanKey As Keys(,) = {{Keys.ShiftKey, Keys.ControlKey, Keys.Menu, pureKey},
                                  {Keys.Shift, Keys.Control, Keys.Alt, pureKey}} '(0,n)=[GetAsyncKeyState]APIで調べるときのキーコード / (1,n)=調べるキーコード

        For i As Byte = 0 To 3 '調べるキーの数 = 修飾子キー3種 + 純粋キー1種

            '※重要！！
            '[PressStatus.Last]ビットは、[GetAsyncKeyState]APIでスキャンするまで残っている。
            'その影響を次回に持ち越さないために、目的のキーが押されていないと判明したとしても、最後までループを続ける必要がある。

            apiReturn = GetAsyncKeyState(scanKey(0, i))

            If apiReturn <> PressStatus.Off Then
                state = state Or scanKey(1, i)
            End If

        Next

        '=== 結果を返す ===
        If TargetKeySet <> state Then
            Return PressStatus.Off
        Else
            Return apiReturn
        End If

    End Function

End Class
